@@ -1,4 +1,4 @@
-/*	$OpenBSD: extern.h,v 1.175 2023/03/13 19:51:49 job Exp $ */
+/*	$OpenBSD: extern.h,v 1.180 2023/04/27 08:37:53 beck Exp $ */
 /*
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -217,6 +217,7 @@ struct mft {
 	time_t		 expires; /* when the signature path expires */
 	size_t		 filesz; /* number of filenames */
 	unsigned int	 repoid;
+	int		 talid;
 	int		 stale; /* if a stale manifest */
 };
 
@@ -383,6 +384,8 @@ struct vap {
 	struct aspa_provider	*providers;
 	size_t			 providersz;
 	time_t			 expires;
+	int			 talid;
+	unsigned int		 repoid;
 };
 
 /*
@@ -397,12 +400,12 @@ RB_PROTOTYPE(vap_tree, vap, entry, vapcmp);
 struct vrp {
 	RB_ENTRY(vrp)	entry;
 	struct ip_addr	addr;
-	int		talid; /* covered by which TAL */
-	unsigned int	repoid;
 	uint32_t	asid;
 	enum afi	afi;
 	unsigned char	maxlength;
 	time_t		expires; /* transitive expiry moment */
+	int		talid; /* covered by which TAL */
+	unsigned int	repoid;
 };
 /*
  * Tree of VRP sorted by afi, addr, maxlength and asid
@@ -507,6 +510,7 @@ struct entity {
 	TAILQ_ENTRY(entity) entries;
 	char		*path;		/* path relative to repository */
 	char		*file;		/* filename or valid repo path */
+	char		*mftaki;	/* expected AKI (taken from Manifest) */
 	unsigned char	*data;		/* optional data blob */
 	size_t		 datasz;	/* length of optional data blob */
 	unsigned int	 repoid;	/* repository identifier */
@@ -538,7 +542,7 @@ RB_HEAD(filepath_tree, filepath);
 /*
  * Statistics collected during run-time.
  */
-struct repostats {
+struct repotalstats {
 	uint32_t	 certs; /* certificates */
 	uint32_t	 certs_fail; /* invalid certificate */
 	uint32_t	 mfts; /* total number of manifests */
@@ -561,6 +565,13 @@ struct repostats {
 	uint32_t	 vaps_pas6; /* total number of IPv6 only providers */
 	uint32_t	 vrps; /* total number of Validated ROA Payloads */
 	uint32_t	 vrps_uniqs; /* number of unique vrps */
+};
+
+struct repostats {
+	uint32_t	 del_files;	/* number of files removed in cleanup */
+	uint32_t	 extra_files;	/* number of superfluous files */
+	uint32_t	 del_extra_files;/* number of removed extra files */
+	uint32_t	 del_dirs;	/* number of dirs removed in cleanup */
 	struct timespec	 sync_time;	/* time to sync repo */
 };
 
@@ -573,11 +584,9 @@ struct stats {
 	uint32_t	 http_fails; /* failed http repositories */
 	uint32_t	 rrdp_repos; /* synced rrdp repositories */
 	uint32_t	 rrdp_fails; /* failed rrdp repositories */
-	uint32_t	 del_files; /* number of files removed in cleanup */
-	uint32_t	 extra_files; /* number of superfluous files */
-	uint32_t	 del_dirs; /* number of dirs removed in cleanup */
 	uint32_t	 skiplistentries; /* number of skiplist entries */
 
+	struct repotalstats	repo_tal_stats;
 	struct repostats	repo_stats;
 	struct timespec		elapsed_time;
 	struct timespec		user_time;
@@ -594,7 +603,7 @@ extern int excludeaspa;
 extern const char *tals[];
 extern const char *taldescs[];
 extern unsigned int talrepocnt[];
-extern struct repostats talstats[];
+extern struct repotalstats talstats[];
 extern int talsz;
 
 /* Routines for RPKI entities. */
@@ -666,7 +675,7 @@ void		 crl_tree_free(struct crl_tree *);
 /* Validation of our objects. */
 
 struct auth	*valid_ski_aki(const char *, struct auth_tree *,
-		    const char *, const char *);
+		    const char *, const char *, const char *);
 int		 valid_ta(const char *, struct auth_tree *,
 		    const struct cert *);
 int		 valid_cert(const char *, struct auth *, const struct cert *);
@@ -756,6 +765,7 @@ const char	*repo_uri(const struct repo *);
 void		 repo_fetch_uris(const struct repo *, const char **,
 		    const char **);
 int		 repo_synced(const struct repo *);
+const char	*repo_proto(const struct repo *);
 int		 repo_talid(const struct repo *);
 struct repo	*ta_lookup(int, struct tal *);
 struct repo	*repo_lookup(int, const char *, const char *);
@@ -763,7 +773,9 @@ struct repo	*repo_byid(unsigned int);
 int		 repo_queued(struct repo *, struct entity *);
 void		 repo_cleanup(struct filepath_tree *, int);
 int		 repo_check_timeout(int);
-void		 repo_stat_inc(struct repo *, enum rtype, enum stype);
+void		 repo_stat_inc(struct repo *, int, enum rtype, enum stype);
+void		 repo_tal_stats_collect(void (*)(const struct repo *,
+		    const struct repotalstats *, void *), int, void *);
 void		 repo_stats_collect(void (*)(const struct repo *,
 		    const struct repostats *, void *), void *);
 void		 repo_free(void);
@@ -949,4 +961,12 @@ int	mkpathat(int, const char *);
 /* Maximum number of delta files per RRDP notification file. */
 #define MAX_RRDP_DELTAS		300
 
+/*
+ * Time - Evaluation time is used as the current time if it is
+ * larger than X509_TIME_MIN, otherwise the system time is used.
+ */
+#define X509_TIME_MAX 253402300799LL
+#define X509_TIME_MIN -62167219200LL
+extern int64_t evaluation_time;
+
 #endif /* ! EXTERN_H */
