@@ -1,4 +1,4 @@
-/*	$OpenBSD: gbr.c,v 1.20 2022/11/30 09:12:50 job Exp $ */
+/*	$OpenBSD: gbr.c,v 1.26 2023/03/12 11:46:35 tb Exp $ */
 /*
  * Copyright (c) 2020 Claudio Jeker <claudio@openbsd.org>
  *
@@ -45,16 +45,18 @@ gbr_parse(X509 **x509, const char *fn, const unsigned char *der, size_t len)
 	struct parse	 p;
 	size_t		 cmsz;
 	unsigned char	*cms;
+	time_t		 signtime = 0;
 
 	memset(&p, 0, sizeof(struct parse));
 	p.fn = fn;
 
-	cms = cms_parse_validate(x509, fn, der, len, gbr_oid, &cmsz);
+	cms = cms_parse_validate(x509, fn, der, len, gbr_oid, &cmsz, &signtime);
 	if (cms == NULL)
 		return NULL;
 
 	if ((p.res = calloc(1, sizeof(*p.res))) == NULL)
 		err(1, NULL);
+	p.res->signtime = signtime;
 	if ((p.res->vcard = strndup(cms, cmsz)) == NULL)
 		err(1, NULL);
 	free(cms);
@@ -74,6 +76,11 @@ gbr_parse(X509 **x509, const char *fn, const unsigned char *der, size_t len)
 		goto out;
 	}
 
+	if (!x509_get_notbefore(*x509, fn, &p.res->notbefore))
+		goto out;
+	if (!x509_get_notafter(*x509, fn, &p.res->notafter))
+		goto out;
+
 	if (!x509_inherits(*x509)) {
 		warnx("%s: RFC 3779 extension not set to inherit", fn);
 		goto out;
