@@ -1,4 +1,4 @@
-/*	$OpenBSD: main.c,v 1.232 2023/02/23 09:50:40 claudio Exp $ */
+/*	$OpenBSD: main.c,v 1.236 2023/04/27 08:37:53 beck Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -51,7 +51,7 @@
 const char	*tals[TALSZ_MAX];
 const char	*taldescs[TALSZ_MAX];
 unsigned int	 talrepocnt[TALSZ_MAX];
-struct repostats talstats[TALSZ_MAX];
+struct repotalstats talstats[TALSZ_MAX];
 int		 talsz;
 
 size_t	entity_queue;
@@ -74,6 +74,8 @@ int	rrdpon = 1;
 int	repo_timeout;
 time_t	deadline;
 
+int64_t  evaluation_time;
+
 struct stats	 stats;
 
 struct fqdnlistentry {
@@ -119,6 +121,7 @@ entity_free(struct entity *ent)
 
 	free(ent->path);
 	free(ent->file);
+	free(ent->mftaki);
 	free(ent->data);
 	free(ent);
 }
@@ -137,6 +140,7 @@ entity_read_req(struct ibuf *b, struct entity *ent)
 	io_read_buf(b, &ent->talid, sizeof(ent->talid));
 	io_read_str(b, &ent->path);
 	io_read_str(b, &ent->file);
+	io_read_str(b, &ent->mftaki);
 	io_read_buf_alloc(b, (void **)&ent->data, &ent->datasz);
 }
 
@@ -156,6 +160,7 @@ entity_write_req(const struct entity *ent)
 	io_simple_buffer(b, &ent->talid, sizeof(ent->talid));
 	io_str_buffer(b, ent->path);
 	io_str_buffer(b, ent->file);
+	io_str_buffer(b, ent->mftaki);
 	io_buf_buffer(b, ent->data, ent->datasz);
 	io_close_buffer(&procq, b);
 }
@@ -180,7 +185,8 @@ entity_write_repo(struct repo *rp)
 	io_simple_buffer(b, &talid, sizeof(talid));
 	io_str_buffer(b, path);
 	io_str_buffer(b, altpath);
-	io_buf_buffer(b, NULL, 0);
+	io_buf_buffer(b, NULL, 0); /* ent->mftaki */
+	io_buf_buffer(b, NULL, 0); /* ent->data */
 	io_close_buffer(&procq, b);
 	free(path);
 	free(altpath);
@@ -209,7 +215,8 @@ entityq_flush(struct entityq *q, struct repo *rp)
  */
 static void
 entityq_add(char *path, char *file, enum rtype type, enum location loc,
-    struct repo *rp, unsigned char *data, size_t datasz, int talid)
+    struct repo *rp, unsigned char *data, size_t datasz, int talid,
+    char *mftaki)
 {
 	struct entity	*p;
 
@@ -219,6 +226,7 @@ entityq_add(char *path, char *file, enum rtype type, enum location loc,
 	p->type = type;
 	p->location = loc;
 	p->talid = talid;
+	p->mftaki = mftaki;
 	p->path = path;
 	if (rp != NULL)
 		p->repoid = repo_id(rp);
@@ -377,7 +385,7 @@ queue_add_from_mft(const struct mft *mft)
 	size_t			 i;
 	struct repo		*rp;
 	const struct mftfile	*f;
-	char			*nfile, *npath = NULL;
+	char			*mftaki, *nfile, *npath = NULL;
 
 	rp = repo_byid(mft->repoid);
 	for (i = 0; i < mft->filesz; i++) {
@@ -391,8 +399,10 @@ queue_add_from_mft(const struct mft *mft)
 				err(1, NULL);
 		if ((nfile = strdup(f->file)) == NULL)
 			err(1, NULL);
+		if ((mftaki = strdup(mft->aki)) == NULL)
+			err(1, NULL);
 		entityq_add(npath, nfile, f->type, f->location, rp, NULL, 0,
-		    -1);
+		    mft->talid, mftaki);
 	}
 }
 
@@ -415,7 +425,8 @@ queue_add_file(const char *file, enum rtype type, int talid)
 	if ((nfile = strdup(file)) == NULL)
 		err(1, NULL);
 	/* Not in a repository, so directly add to queue. */
-	entityq_add(NULL, nfile, type, DIR_UNKNOWN, NULL, buf, len, talid);
+	entityq_add(NULL, nfile, type, DIR_UNKNOWN, NULL, buf, len, talid,
+	    NULL);
 }
 
 /*
@@ -450,7 +461,7 @@ queue_add_from_tal(struct tal *tal)
 	data = tal->pkey;
 	tal->pkey = NULL;
 	entityq_add(NULL, nfile, RTYPE_CER, DIR_VALID, repo, data,
-	    tal->pkeysz, tal->id);
+	    tal->pkeysz, tal->id, NULL);
 }
 
 /*
@@ -518,7 +529,8 @@ queue_add_from_cert(const struct cert *cert)
 			err(1, NULL);
 	}
 
-	entityq_add(npath, nfile, RTYPE_MFT, DIR_UNKNOWN, repo, NULL, 0, -1);
+	entityq_add(npath, nfile, RTYPE_MFT, DIR_UNKNOWN, repo, NULL, 0,
+	    cert->talid, NULL);
 }
 
 /*
@@ -540,6 +552,7 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 	struct repo	*rp;
 	char		*file;
 	unsigned int	 id;
+	int		 talid;
 	int		 c;
 
 	/*
@@ -550,6 +563,7 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 	 */
 	io_read_buf(b, &type, sizeof(type));
 	io_read_buf(b, &id, sizeof(id));
+	io_read_buf(b, &talid, sizeof(talid));
 	io_read_str(b, &file);
 
 	/* in filemode messages can be ignored, only the accounting matters */
@@ -562,7 +576,7 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 	}
 
 	rp = repo_byid(id);
-	repo_stat_inc(rp, type, STYPE_OK);
+	repo_stat_inc(rp, talid, type, STYPE_OK);
 	switch (type) {
 	case RTYPE_TAL:
 		st->tals++;
@@ -573,7 +587,7 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 	case RTYPE_CER:
 		io_read_buf(b, &c, sizeof(c));
 		if (c == 0) {
-			repo_stat_inc(rp, type, STYPE_FAIL);
+			repo_stat_inc(rp, talid, type, STYPE_FAIL);
 			break;
 		}
 		cert = cert_read(b);
@@ -583,7 +597,7 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 			break;
 		case CERT_PURPOSE_BGPSEC_ROUTER:
 			cert_insert_brks(brktree, cert);
-			repo_stat_inc(rp, type, STYPE_BGPSEC);
+			repo_stat_inc(rp, talid, type, STYPE_BGPSEC);
 			break;
 		default:
 			errx(1, "unexpected cert purpose received");
@@ -594,14 +608,14 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 	case RTYPE_MFT:
 		io_read_buf(b, &c, sizeof(c));
 		if (c == 0) {
-			repo_stat_inc(rp, type, STYPE_FAIL);
+			repo_stat_inc(rp, talid, type, STYPE_FAIL);
 			break;
 		}
 		mft = mft_read(b);
 		if (!mft->stale)
 			queue_add_from_mft(mft);
 		else
-			repo_stat_inc(rp, type, STYPE_STALE);
+			repo_stat_inc(rp, talid, type, STYPE_STALE);
 		mft_free(mft);
 		break;
 	case RTYPE_CRL:
@@ -611,14 +625,14 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 	case RTYPE_ROA:
 		io_read_buf(b, &c, sizeof(c));
 		if (c == 0) {
-			repo_stat_inc(rp, type, STYPE_FAIL);
+			repo_stat_inc(rp, talid, type, STYPE_FAIL);
 			break;
 		}
 		roa = roa_read(b);
 		if (roa->valid)
 			roa_insert_vrps(tree, roa, rp);
 		else
-			repo_stat_inc(rp, type, STYPE_INVALID);
+			repo_stat_inc(rp, talid, type, STYPE_INVALID);
 		roa_free(roa);
 		break;
 	case RTYPE_GBR:
@@ -626,14 +640,14 @@ entity_process(struct ibuf *b, struct stats *st, struct vrp_tree *tree,
 	case RTYPE_ASPA:
 		io_read_buf(b, &c, sizeof(c));
 		if (c == 0) {
-			repo_stat_inc(rp, type, STYPE_FAIL);
+			repo_stat_inc(rp, talid, type, STYPE_FAIL);
 			break;
 		}
 		aspa = aspa_read(b);
 		if (aspa->valid)
 			aspa_insert_vaps(vaptree, aspa, rp);
 		else
-			repo_stat_inc(rp, type, STYPE_INVALID);
+			repo_stat_inc(rp, talid, type, STYPE_INVALID);
 		aspa_free(aspa);
 		break;
 	case RTYPE_TAK:
@@ -706,12 +720,9 @@ rrdp_process(struct ibuf *b)
 }
 
 static void
-sum_stats(const struct repo *rp, const struct repostats *in, void *arg)
+sum_stats(const struct repo *rp, const struct repotalstats *in, void *arg)
 {
-	struct repostats *out = arg;
-
-	if (rp != NULL)
-		sum_stats(NULL, in, &talstats[repo_talid(rp)]);
+	struct repotalstats *out = arg;
 
 	out->mfts += in->mfts;
 	out->mfts_fail += in->mfts_fail;
@@ -735,7 +746,16 @@ sum_stats(const struct repo *rp, const struct repostats *in, void *arg)
 	out->vaps_pas += in->vaps_pas;
 	out->vaps_pas4 += in->vaps_pas4;
 	out->vaps_pas6 += in->vaps_pas6;
+}
 
+static void
+sum_repostats(const struct repo *rp, const struct repostats *in, void *arg)
+{
+	struct repostats *out = arg;
+
+	out->del_files += in->del_files;
+	out->extra_files += in->extra_files;
+	out->del_dirs += in->del_dirs;
 	timespecadd(&in->sync_time, &out->sync_time, &out->sync_time);
 }
 
@@ -945,7 +965,9 @@ main(int argc, char *argv[])
 	    "proc exec unveil", NULL) == -1)
 		err(1, "pledge");
 
-	while ((c = getopt(argc, argv, "Ab:Bcd:e:fH:jmnorRs:S:t:T:vV")) != -1)
+	evaluation_time = time(NULL);
+
+	while ((c = getopt(argc, argv, "Ab:Bcd:e:fH:jmnoP:rRs:S:t:T:vV")) != -1)
 		switch (c) {
 		case 'A':
 			excludeaspa = 1;
@@ -985,6 +1007,12 @@ main(int argc, char *argv[])
 		case 'o':
 			outformats |= FORMAT_OPENBGPD;
 			break;
+		case 'P':
+			evaluation_time = strtonum(optarg, X509_TIME_MIN,
+			    X509_TIME_MAX, &errs);
+			if (errs)
+				errx(1, "-P: time in seconds %s", errs);
+			break;
 		case 'R':
 			rrdpon = 0;
 			break;
@@ -1388,7 +1416,11 @@ main(int argc, char *argv[])
 	if (fchdir(outdirfd) == -1)
 		err(1, "fchdir output dir");
 
-	repo_stats_collect(sum_stats, &stats.repo_stats);
+	for (i = 0; i < talsz; i++) {
+		repo_tal_stats_collect(sum_stats, i, &talstats[i]);
+		repo_tal_stats_collect(sum_stats, i, &stats.repo_tal_stats);
+	}
+	repo_stats_collect(sum_repostats, &stats.repo_stats);
 
 	if (outputfiles(&vrps, &brks, &vaps, &stats))
 		rc = 1;
@@ -1400,29 +1432,32 @@ main(int argc, char *argv[])
 	    (long long)stats.system_time.tv_sec);
 	printf("Skiplist entries: %u\n", stats.skiplistentries);
 	printf("Route Origin Authorizations: %u (%u failed parse, %u "
-	    "invalid)\n", stats.repo_stats.roas, stats.repo_stats.roas_fail,
-	    stats.repo_stats.roas_invalid);
+	    "invalid)\n", stats.repo_tal_stats.roas,
+	    stats.repo_tal_stats.roas_fail,
+	    stats.repo_tal_stats.roas_invalid);
 	printf("AS Provider Attestations: %u (%u failed parse, %u "
-	    "invalid)\n", stats.repo_stats.aspas, stats.repo_stats.aspas_fail,
-	    stats.repo_stats.aspas_invalid);
-	printf("BGPsec Router Certificates: %u\n", stats.repo_stats.brks);
+	    "invalid)\n", stats.repo_tal_stats.aspas,
+	    stats.repo_tal_stats.aspas_fail,
+	    stats.repo_tal_stats.aspas_invalid);
+	printf("BGPsec Router Certificates: %u\n", stats.repo_tal_stats.brks);
 	printf("Certificates: %u (%u invalid)\n",
-	    stats.repo_stats.certs, stats.repo_stats.certs_fail);
+	    stats.repo_tal_stats.certs, stats.repo_tal_stats.certs_fail);
 	printf("Trust Anchor Locators: %u (%u invalid)\n",
 	    stats.tals, talsz - stats.tals);
 	printf("Manifests: %u (%u failed parse, %u stale)\n",
-	    stats.repo_stats.mfts, stats.repo_stats.mfts_fail,
-	    stats.repo_stats.mfts_stale);
-	printf("Certificate revocation lists: %u\n", stats.repo_stats.crls);
-	printf("Ghostbuster records: %u\n", stats.repo_stats.gbrs);
-	printf("Trust Anchor Keys: %u\n", stats.repo_stats.taks);
+	    stats.repo_tal_stats.mfts, stats.repo_tal_stats.mfts_fail,
+	    stats.repo_tal_stats.mfts_stale);
+	printf("Certificate revocation lists: %u\n", stats.repo_tal_stats.crls);
+	printf("Ghostbuster records: %u\n", stats.repo_tal_stats.gbrs);
+	printf("Trust Anchor Keys: %u\n", stats.repo_tal_stats.taks);
 	printf("Repositories: %u\n", stats.repos);
 	printf("Cleanup: removed %u files, %u directories, %u superfluous\n",
-	    stats.del_files, stats.del_dirs, stats.extra_files);
-	printf("VRP Entries: %u (%u unique)\n", stats.repo_stats.vrps,
-	    stats.repo_stats.vrps_uniqs);
-	printf("VAP Entries: %u (%u unique)\n", stats.repo_stats.vaps,
-	    stats.repo_stats.vaps_uniqs);
+	    stats.repo_stats.del_files, stats.repo_stats.del_dirs,
+	    stats.repo_stats.extra_files);
+	printf("VRP Entries: %u (%u unique)\n", stats.repo_tal_stats.vrps,
+	    stats.repo_tal_stats.vrps_uniqs);
+	printf("VAP Entries: %u (%u unique)\n", stats.repo_tal_stats.vaps,
+	    stats.repo_tal_stats.vaps_uniqs);
 
 	/* Memory cleanup. */
 	repo_free();
