@@ -1,4 +1,4 @@
-/*	$OpenBSD: repo.c,v 1.39 2022/09/02 21:56:45 claudio Exp $ */
+/*	$OpenBSD: repo.c,v 1.44 2023/04/26 16:32:41 claudio Exp $ */
 /*
  * Copyright (c) 2021 Claudio Jeker <claudio@openbsd.org>
  * Copyright (c) 2019 Kristaps Dzonsons <kristaps@bsd.lv>
@@ -97,8 +97,12 @@ struct repo {
 	const struct rsyncrepo	*rsync;
 	const struct tarepo	*ta;
 	struct entityq		 queue;		/* files waiting for repo */
+	struct repotalstats	 stats[TALSZ_MAX];
+	struct repostats	 repostats;
+	struct timespec		 start_time;
 	time_t			 alarm;		/* sync timeout */
 	int			 talid;
+	int			 stats_used[TALSZ_MAX];
 	unsigned int		 id;		/* identifier */
 };
 static SLIST_HEAD(, repo)	repos = SLIST_HEAD_INITIALIZER(repos);
@@ -263,7 +267,7 @@ repo_mkpath(int fd, char *file)
  * Return the state of a repository.
  */
 static enum repo_state
-repo_state(struct repo *rp)
+repo_state(const struct repo *rp)
 {
 	if (rp->ta)
 		return rp->ta->state;
@@ -283,24 +287,25 @@ static void
 repo_done(const void *vp, int ok)
 {
 	struct repo *rp;
+	struct timespec flush_time;
 
 	SLIST_FOREACH(rp, &repos, entry) {
-		if (vp == rp->ta)
-			entityq_flush(&rp->queue, rp);
-		if (vp == rp->rsync)
-			entityq_flush(&rp->queue, rp);
-		if (vp == rp->rrdp) {
-			if (!ok && !nofetch) {
-				/* try to fall back to rsync */
-				rp->rrdp = NULL;
-				rp->rsync = rsync_get(rp->repouri,
-				    rp->basedir);
-				/* need to check if it was already loaded */
-				if (repo_state(rp) != REPO_LOADING)
-					entityq_flush(&rp->queue, rp);
-			} else
-				entityq_flush(&rp->queue, rp);
+		if (vp != rp->ta && vp != rp->rsync && vp != rp->rrdp)
+			continue;
+
+		/* for rrdp try to fall back to rsync */
+		if (vp == rp->rrdp && !ok && !nofetch) {
+			rp->rrdp = NULL;
+			rp->rsync = rsync_get(rp->repouri, rp->basedir);
+			/* need to check if it was already loaded */
+			if (repo_state(rp) == REPO_LOADING)
+				continue;
 		}
+
+		entityq_flush(&rp->queue, rp);
+		clock_gettime(CLOCK_MONOTONIC, &flush_time);
+		timespecsub(&flush_time, &rp->start_time,
+		    &rp->repostats.sync_time);
 	}
 }
 
@@ -553,16 +558,18 @@ rrdp_free(void)
  * Check if a directory is an active rrdp repository.
  * Returns 1 if found else 0.
  */
-static int
-rrdp_is_active(const char *dir)
+static struct repo *
+repo_rrdp_bypath(const char *dir)
 {
-	struct rrdprepo *rr;
+	struct repo *rp;
 
-	SLIST_FOREACH(rr, &rrdprepos, entry)
-		if (strcmp(dir, rr->basedir) == 0)
-			return rr->state != REPO_FAILED;
-
-	return 0;
+	SLIST_FOREACH(rp, &repos, entry) {
+		if (rp->rrdp == NULL)
+			continue;
+		if (strcmp(dir, rp->rrdp->basedir) == 0)
+			return rp;
+	}
+	return NULL;
 }
 
 /*
@@ -600,6 +607,7 @@ repo_alloc(int talid)
 	rp->alarm = getmonotime() + repo_timeout;
 	TAILQ_INIT(&rp->queue);
 	SLIST_INSERT_HEAD(&repos, rp, entry);
+	clock_gettime(CLOCK_MONOTONIC, &rp->start_time);
 
 	stats.repos++;
 	return rp;
@@ -681,7 +689,7 @@ rrdp_save_state(unsigned int id, struct rrdp_session *state)
 
 	rr = rrdp_find(id);
 	if (rr == NULL)
-		errx(1, "non-existant rrdp repo %u", id);
+		errx(1, "non-existent rrdp repo %u", id);
 
 	file = rrdp_state_filename(rr, 0);
 	temp = rrdp_state_filename(rr, 1);
@@ -777,7 +785,7 @@ rrdp_clear(unsigned int id)
 
 	rr = rrdp_find(id);
 	if (rr == NULL)
-		errx(1, "non-existant rrdp repo %u", id);
+		errx(1, "non-existent rrdp repo %u", id);
 
 	/* remove rrdp repository contents */
 	remove_contents(rr->basedir);
@@ -801,7 +809,7 @@ rrdp_handle_file(unsigned int id, enum publish_type pt, char *uri,
 
 	rr = rrdp_find(id);
 	if (rr == NULL)
-		errx(1, "non-existant rrdp repo %u", id);
+		errx(1, "non-existent rrdp repo %u", id);
 	if (rr->state == REPO_FAILED)
 		return -1;
 
@@ -920,7 +928,7 @@ rsync_finish(unsigned int id, int ok)
 }
 
 /*
- * RRDP sync finshed, either with or without success.
+ * RRDP sync finished, either with or without success.
  */
 void
 rrdp_finish(unsigned int id, int ok)
@@ -964,7 +972,7 @@ http_finish(unsigned int id, enum http_result res, const char *last_mod)
 
 	tr = ta_find(id);
 	if (tr == NULL) {
-		/* not a TA fetch therefor RRDP */
+		/* not a TA fetch therefore RRDP */
 		rrdp_http_done(id, res, last_mod);
 		return;
 	}
@@ -1009,18 +1017,18 @@ ta_lookup(int id, struct tal *tal)
 
 	/* Look up in repository table. (Lookup should actually fail here) */
 	SLIST_FOREACH(rp, &repos, entry) {
-		if (strcmp(rp->repouri, tal->descr) == 0)
+		if (strcmp(rp->repouri, tal->uri[0]) == 0)
 			return rp;
 	}
 
 	rp = repo_alloc(id);
 	rp->basedir = repo_dir(tal->descr, "ta", 0);
-	if ((rp->repouri = strdup(tal->descr)) == NULL)
+	if ((rp->repouri = strdup(tal->uri[0])) == NULL)
 		err(1, NULL);
 
 	/* check if sync disabled ... */
 	if (noop) {
-		logx("ta/%s: using cache", rp->repouri);
+		logx("%s: using cache", rp->basedir);
 		entityq_flush(&rp->queue, rp);
 		return rp;
 	}
@@ -1179,6 +1187,59 @@ repo_uri(const struct repo *rp)
 	return rp->repouri;
 }
 
+/*
+ * Return the repository URI.
+ */
+void
+repo_fetch_uris(const struct repo *rp, const char **carepo,
+    const char **notifyuri)
+{
+	*carepo = rp->repouri;
+	*notifyuri = rp->notifyuri;
+}
+
+/*
+ * Return 1 if repository is synced else 0.
+ */
+int
+repo_synced(const struct repo *rp)
+{
+	if (repo_state(rp) == REPO_DONE &&
+	    !(rp->rrdp == NULL && rp->rsync == NULL && rp->ta == NULL))
+		return 1;
+	return 0;
+}
+
+/*
+ * Return the protocol string "rrdp", "rsync", "https" which was used to sync.
+ * Result is only correct if repository was properly synced.
+ */
+const char *
+repo_proto(const struct repo *rp)
+{
+
+	if (rp->ta != NULL) {
+		const struct tarepo *tr = rp->ta;
+		if (tr->uriidx < tr->urisz && 
+		    strncasecmp(tr->uri[tr->uriidx], "rsync://", 8) == 0)
+			return "rsync";
+		else
+			return "https";
+	}
+	if (rp->rrdp != NULL)
+		return "rrdp";
+	return "rsync";
+}
+
+/*
+ * Return the repository tal ID.
+ */
+int
+repo_talid(const struct repo *rp)
+{
+	return rp->talid;
+}
+
 int
 repo_queued(struct repo *rp, struct entity *p)
 {
@@ -1262,6 +1323,127 @@ repo_check_timeout(int timeout)
 	return timeout;
 }
 
+/*
+ * Update stats object of repository depending on rtype and subtype.
+ */
+void
+repo_stat_inc(struct repo *rp, int talid, enum rtype type, enum stype subtype)
+{
+	if (rp == NULL)
+		return;
+	rp->stats_used[talid] = 1;
+	switch (type) {
+	case RTYPE_CER:
+		if (subtype == STYPE_OK)
+			rp->stats[talid].certs++;
+		if (subtype == STYPE_FAIL)
+			rp->stats[talid].certs_fail++;
+		if (subtype == STYPE_BGPSEC) {
+			rp->stats[talid].certs--;
+			rp->stats[talid].brks++;
+		}
+		break;
+	case RTYPE_MFT:
+		if (subtype == STYPE_OK)
+			rp->stats[talid].mfts++;
+		if (subtype == STYPE_FAIL)
+			rp->stats[talid].mfts_fail++;
+		if (subtype == STYPE_STALE)
+			rp->stats[talid].mfts_stale++;
+		break;
+	case RTYPE_ROA:
+		switch (subtype) {
+		case STYPE_OK:
+			rp->stats[talid].roas++;
+			break;
+		case STYPE_FAIL:
+			rp->stats[talid].roas_fail++;
+			break;
+		case STYPE_INVALID:
+			rp->stats[talid].roas_invalid++;
+			break;
+		case STYPE_TOTAL:
+			rp->stats[talid].vrps++;
+			break;
+		case STYPE_UNIQUE:
+			rp->stats[talid].vrps_uniqs++;
+			break;
+		case STYPE_DEC_UNIQUE:
+			rp->stats[talid].vrps_uniqs--;
+			break;
+		default:
+			break;
+		}
+		break;
+	case RTYPE_ASPA:
+		switch (subtype) {
+		case STYPE_OK:
+			rp->stats[talid].aspas++;
+			break;
+		case STYPE_FAIL:
+			rp->stats[talid].aspas_fail++;
+			break;
+		case STYPE_INVALID:
+			rp->stats[talid].aspas_invalid++;
+			break;
+		case STYPE_TOTAL:
+			rp->stats[talid].vaps++;
+			break;
+		case STYPE_UNIQUE:
+			rp->stats[talid].vaps_uniqs++;
+			break;
+		case STYPE_DEC_UNIQUE:
+			rp->stats[talid].vaps_uniqs--;
+			break;
+		case STYPE_BOTH:
+			rp->stats[talid].vaps_pas++;
+			break;
+		case STYPE_ONLY_IPV4:
+			rp->stats[talid].vaps_pas4++;
+			break;
+		case STYPE_ONLY_IPV6:
+			rp->stats[talid].vaps_pas6++;
+			break;
+		default:
+			break;
+		}
+		break;
+	case RTYPE_CRL:
+		rp->stats[talid].crls++;
+		break;
+	case RTYPE_GBR:
+		rp->stats[talid].gbrs++;
+		break;
+	case RTYPE_TAK:
+		rp->stats[talid].taks++;
+		break;
+	default:
+		break;
+	}
+}
+
+void
+repo_tal_stats_collect(void (*cb)(const struct repo *,
+    const struct repotalstats *, void *), int talid, void *arg)
+{
+	struct repo	*rp;
+
+	SLIST_FOREACH(rp, &repos, entry) {
+		if (rp->stats_used[talid])
+			cb(rp, &rp->stats[talid], arg);
+	}
+}
+
+void
+repo_stats_collect(void (*cb)(const struct repo *, const struct repostats *,
+    void *), void *arg)
+{
+	struct repo	*rp;
+
+	SLIST_FOREACH(rp, &repos, entry)
+		cb(rp, &rp->repostats, arg);
+}
+
 /*
  * Delayed delete of files from RRDP. Since RRDP has no security built-in
  * this code needs to check if this RRDP repository is actually allowed to
@@ -1270,11 +1452,15 @@ repo_check_timeout(int timeout)
 static void
 repo_cleanup_rrdp(struct filepath_tree *tree)
 {
+	struct repo *rp;
 	struct rrdprepo *rr;
 	struct filepath *fp, *nfp;
 	char *fn;
 
-	SLIST_FOREACH(rr, &rrdprepos, entry) {
+	SLIST_FOREACH(rp, &repos, entry) {
+		if (rp->rrdp == NULL)
+			continue;
+		rr = (struct rrdprepo *)rp->rrdp;
 		RB_FOREACH_SAFE(fp, filepath_tree, &rr->deleted, nfp) {
 			if (!rrdp_uri_valid(rr, fp->file)) {
 				warnx("%s: external URI %s", rr->notifyuri,
@@ -1291,7 +1477,7 @@ repo_cleanup_rrdp(struct filepath_tree *tree)
 			} else {
 				if (verbose > 1)
 					logx("deleted %s", fn);
-				stats.del_files++;
+				rp->repostats.del_files++;
 			}
 			free(fn);
 
@@ -1304,7 +1490,7 @@ repo_cleanup_rrdp(struct filepath_tree *tree)
 				} else {
 					if (verbose > 1)
 						logx("deleted %s", fn);
-					stats.del_files++;
+					rp->repostats.del_files++;
 				}
 			} else
 				warnx("%s: referenced file supposed to be "
@@ -1362,18 +1548,16 @@ repo_move_valid(struct filepath_tree *tree)
 	}
 }
 
-#define	BASE_DIR	(void *)0x01
-#define	RSYNC_DIR	(void *)0x02
-#define	RRDP_DIR	(void *)0x03
+struct fts_state {
+	enum { BASE_DIR, RSYNC_DIR, RRDP_DIR }	type;
+	struct repo				*rp;
+} fts_state;
 
 static const struct rrdprepo *
 repo_is_rrdp(struct repo *rp)
 {
 	/* check for special pointers first these are not a repository */
-	if (rp == NULL || rp == BASE_DIR || rp == RSYNC_DIR || rp == RRDP_DIR)
-		return NULL;
-
-	if (rp->rrdp)
+	if (rp != NULL && rp->rrdp != NULL)
 		return rp->rrdp->state == REPO_DONE ? rp->rrdp : NULL;
 	return NULL;
 }
@@ -1386,13 +1570,155 @@ skip_dotslash(char *in)
 	return in;
 }
 
+static void
+repo_cleanup_entry(FTSENT *e, struct filepath_tree *tree, int cachefd)
+{
+	const struct rrdprepo *rr;
+	char *path;
+
+	path = skip_dotslash(e->fts_path);
+	switch (e->fts_info) {
+	case FTS_NSOK:
+		if (filepath_exists(tree, path)) {
+			e->fts_parent->fts_number++;
+			break;
+		}
+		if (fts_state.type == RRDP_DIR && fts_state.rp != NULL) {
+			e->fts_parent->fts_number++;
+			/* handle rrdp .state files explicitly */
+			if (e->fts_level == 3 &&
+			    strcmp(e->fts_name, ".state") == 0)
+				break;
+			/* can't delete these extra files */
+			fts_state.rp->repostats.extra_files++;
+			if (verbose > 1)
+				logx("superfluous %s", path);
+			break;
+		}
+		rr = repo_is_rrdp(fts_state.rp);
+		if (rr != NULL) {
+			struct stat st;
+			char *fn;
+
+			if (asprintf(&fn, "%s/%s", rr->basedir, path) == -1)
+				err(1, NULL);
+
+			/*
+			 * If the file exists in the rrdp dir
+			 * that file is newer and needs to be kept
+			 * so unlink this file instead of moving
+			 * it over the file in the rrdp dir.
+			 */
+			if (fstatat(cachefd, fn, &st, 0) == 0 &&
+			    S_ISREG(st.st_mode)) {
+				free(fn);
+				goto unlink;
+			}
+			if (repo_mkpath(cachefd, fn) == 0) {
+				if (renameat(AT_FDCWD, e->fts_accpath,
+				    cachefd, fn) == -1)
+					warn("rename %s to %s", path, fn);
+				else if (verbose > 1)
+					logx("moved %s", path);
+				fts_state.rp->repostats.extra_files++;
+			}
+			free(fn);
+		} else {
+ unlink:
+			if (unlink(e->fts_accpath) == -1) {
+				warn("unlink %s", path);
+			} else if (fts_state.type == RSYNC_DIR) {
+				/* no need to keep rsync files */
+				if (verbose > 1)
+					logx("superfluous %s", path);
+				if (fts_state.rp != NULL)
+					fts_state.rp->repostats.del_extra_files++;
+				else 
+					stats.repo_stats.del_extra_files++;
+			} else {
+				if (verbose > 1)
+					logx("deleted %s", path);
+				if (fts_state.rp != NULL)
+					fts_state.rp->repostats.del_files++;
+				else 
+					stats.repo_stats.del_files++;
+			}
+		}
+		break;
+	case FTS_D:
+		if (e->fts_level == FTS_ROOTLEVEL)
+			fts_state.type = BASE_DIR;
+		if (e->fts_level == 1) {
+			if (strcmp(".rsync", e->fts_name) == 0) {
+				fts_state.type = RSYNC_DIR;
+				fts_state.rp = NULL;
+			} else if (strcmp(".rrdp", e->fts_name) == 0) {
+				fts_state.type = RRDP_DIR;
+				fts_state.rp = NULL;
+			} else {
+				fts_state.type = BASE_DIR;
+				fts_state.rp = repo_bypath(path);
+			}
+		}
+		if (e->fts_level == 2) {
+			if (fts_state.type == RSYNC_DIR)
+				fts_state.rp = repo_bypath(path);
+			/*
+			 * special handling for rrdp directories,
+			 * clear them if they are not used anymore but
+			 * only if rrdp is active.
+			 */
+			if (fts_state.type == RRDP_DIR)
+				fts_state.rp = repo_rrdp_bypath(path);
+		}
+		break;
+	case FTS_DP:
+		if (e->fts_level == FTS_ROOTLEVEL)
+			break;
+		if (e->fts_level == 1) {
+			/* do not remove .rsync and .rrdp */
+			fts_state.rp = NULL;
+			if (fts_state.type == RRDP_DIR ||
+			    fts_state.type == RSYNC_DIR)
+				break;
+		}
+
+		e->fts_parent->fts_number += e->fts_number;
+
+		if (e->fts_number == 0) {
+			if (rmdir(e->fts_accpath) == -1)
+				warn("rmdir %s", path);
+			if (fts_state.rp != NULL)
+				fts_state.rp->repostats.del_dirs++;
+			else 
+				stats.repo_stats.del_dirs++;
+		}
+		break;
+	case FTS_SL:
+	case FTS_SLNONE:
+		warnx("symlink %s", path);
+		if (unlink(e->fts_accpath) == -1)
+			warn("unlink %s", path);
+		stats.repo_stats.del_extra_files++;
+		break;
+	case FTS_NS:
+	case FTS_ERR:
+		if (e->fts_errno == ENOENT && e->fts_level == FTS_ROOTLEVEL)
+			break;
+		warnx("fts_read %s: %s", path, strerror(e->fts_errno));
+		break;
+	default:
+		warnx("fts_read %s: unhandled[%x]", path, e->fts_info);
+		break;
+	}
+}
+
 void
 repo_cleanup(struct filepath_tree *tree, int cachefd)
 {
 	char *argv[2] = { ".", NULL };
 	FTS *fts;
 	FTSENT *e;
-	const struct rrdprepo *rr;
 
 	/* first move temp files which have been used to valid dir */
 	repo_move_valid(tree);
@@ -1403,135 +1729,7 @@ repo_cleanup(struct filepath_tree *tree, int cachefd)
 		err(1, "fts_open");
 	errno = 0;
 	while ((e = fts_read(fts)) != NULL) {
-		char *path = skip_dotslash(e->fts_path);
-		switch (e->fts_info) {
-		case FTS_NSOK:
-			if (filepath_exists(tree, path)) {
-				e->fts_parent->fts_number++;
-				break;
-			}
-			if (e->fts_parent->fts_pointer == RRDP_DIR) {
-				e->fts_parent->fts_number++;
-				/* handle rrdp .state files explicitly */
-				if (e->fts_level == 3 &&
-				    strcmp(e->fts_name, ".state") == 0)
-					break;
-				/* can't delete these extra files */
-				stats.extra_files++;
-				if (verbose > 1)
-					logx("superfluous %s", path);
-				break;
-			}
-			if (e->fts_parent->fts_pointer == RSYNC_DIR) {
-				/* no need to keep rsync files */
-				if (verbose > 1)
-					logx("superfluous %s", path);
-			}
-			rr = repo_is_rrdp(e->fts_parent->fts_pointer);
-			if (rr != NULL) {
-				struct stat st;
-				char *fn;
-
-				if (asprintf(&fn, "%s/%s", rr->basedir,
-				    path) == -1)
-					err(1, NULL);
-
-				/*
-				 * If the file exists in the rrdp dir
-				 * that file is newer and needs to be kept
-				 * so unlink this file instead of moving
-				 * it over the file in the rrdp dir.
-				 */
-				if (fstatat(cachefd, fn, &st, 0) == 0 &&
-				    S_ISREG(st.st_mode)) {
-					free(fn);
-					goto unlink;
-				}
-				if (repo_mkpath(cachefd, fn) == 0) {
-					if (renameat(AT_FDCWD, e->fts_accpath,
-					    cachefd, fn) == -1)
-						warn("rename %s to %s", path,
-						    fn);
-					else if (verbose > 1)
-						logx("moved %s", path);
-					stats.extra_files++;
-				}
-				free(fn);
-			} else {
- unlink:
-				if (unlink(e->fts_accpath) == -1) {
-					warn("unlink %s", path);
-				} else {
-					if (verbose > 1)
-						logx("deleted %s", path);
-					stats.del_files++;
-				}
-			}
-			break;
-		case FTS_D:
-			if (e->fts_level == 1) {
-				if (strcmp(".rsync", e->fts_name) == 0)
-					e->fts_pointer = RSYNC_DIR;
-				else if (strcmp(".rrdp", e->fts_name) == 0)
-					e->fts_pointer = RRDP_DIR;
-				else
-					e->fts_pointer = BASE_DIR;
-			} else
-				e->fts_pointer = e->fts_parent->fts_pointer;
-
-			/*
-			 * special handling for rrdp directories,
-			 * clear them if they are not used anymore but
-			 * only if rrdp is active.
-			 */
-			if (e->fts_pointer == RRDP_DIR && e->fts_level == 2) {
-				if (!rrdp_is_active(path))
-					e->fts_pointer = NULL;
-			}
-			if (e->fts_pointer == BASE_DIR && e->fts_level > 1) {
-				e->fts_pointer = repo_bypath(path);
-				if (e->fts_pointer == NULL)
-					e->fts_pointer = BASE_DIR;
-			}
-			break;
-		case FTS_DP:
-			if (e->fts_level == FTS_ROOTLEVEL)
-				break;
-			if (e->fts_level == 1)
-				/* do not remove .rsync and .rrdp */
-				if (e->fts_pointer == RRDP_DIR ||
-				    e->fts_pointer == RSYNC_DIR)
-					break;
-
-			e->fts_parent->fts_number += e->fts_number;
-
-			if (e->fts_number == 0) {
-				if (rmdir(e->fts_accpath) == -1)
-					warn("rmdir %s", path);
-				else
-					stats.del_dirs++;
-			}
-			break;
-		case FTS_SL:
-		case FTS_SLNONE:
-			warnx("symlink %s", path);
-			if (unlink(e->fts_accpath) == -1)
-				warn("unlink %s", path);
-			break;
-		case FTS_NS:
-		case FTS_ERR:
-			if (e->fts_errno == ENOENT &&
-			    e->fts_level == FTS_ROOTLEVEL)
-				continue;
-			warnx("fts_read %s: %s", path,
-			    strerror(e->fts_errno));
-			break;
-		default:
-			warnx("fts_read %s: unhandled[%x]", path,
-			    e->fts_info);
-			break;
-		}
-
+		repo_cleanup_entry(e, tree, cachefd);
 		errno = 0;
 	}
 	if (errno)
