@@ -1,4 +1,4 @@
-/*	$OpenBSD: rsc.c,v 1.19 2022/11/26 17:14:40 job Exp $ */
+/*	$OpenBSD: rsc.c,v 1.25 2023/03/12 13:31:39 tb Exp $ */
 /*
  * Copyright (c) 2022 Theo Buehler <tb@openbsd.org>
  * Copyright (c) 2022 Job Snijders <job@fastly.com>
@@ -377,19 +377,21 @@ rsc_parse(X509 **x509, const char *fn, const unsigned char *der, size_t len)
 	struct parse		 p;
 	unsigned char		*cms;
 	size_t			 cmsz;
-	const ASN1_TIME		*at;
 	struct cert		*cert = NULL;
+	time_t			 signtime = 0;
 	int			 rc = 0;
 
 	memset(&p, 0, sizeof(struct parse));
 	p.fn = fn;
 
-	cms = cms_parse_validate(x509, fn, der, len, rsc_oid, &cmsz);
+	cms = cms_parse_validate(x509, fn, der, len, rsc_oid, &cmsz,
+	    &signtime);
 	if (cms == NULL)
 		return NULL;
 
 	if ((p.res = calloc(1, sizeof(struct rsc))) == NULL)
 		err(1, NULL);
+	p.res->signtime = signtime;
 
 	if (!x509_get_aia(*x509, fn, &p.res->aia))
 		goto out;
@@ -403,15 +405,10 @@ rsc_parse(X509 **x509, const char *fn, const unsigned char *der, size_t len)
 		goto out;
 	}
 
-	at = X509_get0_notAfter(*x509);
-	if (at == NULL) {
-		warnx("%s: X509_get0_notAfter failed", fn);
+	if (!x509_get_notbefore(*x509, fn, &p.res->notbefore))
 		goto out;
-	}
-	if (!x509_get_time(at, &p.res->expires)) {
-		warnx("%s: ASN1_time_parse failed", fn);
+	if (!x509_get_notafter(*x509, fn, &p.res->notafter))
 		goto out;
-	}
 
 	if (X509_get_ext_by_NID(*x509, NID_sinfo_access, -1) != -1) {
 		warnx("%s: RSC: EE cert must not have an SIA extension", fn);
